package nl.iobyte.themeparkconnector.api;

import nl.iobyte.themeparkconnector.ThemeParkConnector;
import nl.iobyte.themeparkconnector.api.authentication.AuthenticationService;
import nl.iobyte.themeparkconnector.api.config.ConfigurationManager;
import nl.iobyte.themeparkconnector.api.event.EventDispatcher;
import nl.iobyte.themeparkconnector.api.load.DataLoadService;
import nl.iobyte.themeparkconnector.api.network.NetworkingService;
import nl.iobyte.themeparkconnector.api.operator.OperatorService;
import nl.iobyte.themeparkconnector.api.player.PlayerStateService;
import nl.iobyte.themeparkconnector.api.show.ShowService;
import nl.iobyte.themeparkconnector.api.state.StateService;
import nl.iobyte.themeparkconnector.logger.ThemeParkConnectorLogger;

public class ThemeParkConnectorAPI {

    //Data
    private ConfigurationManager configurationManager;
    private final DataLoadService dataLoadService = new DataLoadService();
    private final ShowService showService = new ShowService();

    //ThemeParkConnector System
    private final NetworkingService networkingService = new NetworkingService();
    private final StateService stateService = new StateService();
    private final PlayerStateService playerStateService = new PlayerStateService();
    private AuthenticationService authenticationService;

    //Attraction
    private final OperatorService operatorService = new OperatorService();

    //General
    private EventDispatcher eventDispatcher;

    /**
     * Enable the API
     */
    public void enable() {
        eventDispatcher = new EventDispatcher(ThemeParkConnector.getInstance());

        //Prepare Configuration
        configurationManager = new ConfigurationManager(ThemeParkConnector.getInstance());

        //Load data
        dataLoadService.init();
        authenticationService = new AuthenticationService();
        networkingService.init();
    }

    /**
     * Disable the API
     */
    public void disable() {
        try {
            if (stateService.isConnected())
                networkingService.stop();
        } catch (NoClassDefFoundError e) {
            ThemeParkConnectorLogger.toConsole("Bukkit already unloaded the networking classes, can't kill socket.");
        }
    }

    /**
     * Get the show service
     * @return ShowService
     */
    public ShowService getShowService() {
        return showService;
    }

    /**
     * Get the networking service
     * @return NetworkingService
     */
    public NetworkingService getNetworkingService() {
        return networkingService;
    }

    /**
     * Get the configuration manager
     * @return ConfigurationManager
     */
    public ConfigurationManager getConfigurationManager() {
        return configurationManager;
    }

    /**
     * Get the state service
     * @return StateService
     */
    public StateService getStateService() {
        return stateService;
    }

    /**
     * Get the player state service
     * @return PlayerStateService
     */
    public PlayerStateService getPlayerStateService() {
        return playerStateService;
    }

    /**
     * Get the data load service
     * @return DataLoadService
     */
    public DataLoadService getDataLoadService() {
        return dataLoadService;
    }

    /**
     * Get the operator service
     * @return OperatorService
     */
    public OperatorService getOperatorService() {
        return operatorService;
    }

    /**
     * Get event dispatcher
     * @return EventDispatcher
     */
    public EventDispatcher getEventDispatcher() {
        return eventDispatcher;
    }

    /**
     * Get authentication service
     * @return AuthenticationService
     */
    public AuthenticationService getAuthenticationService() {
        return authenticationService;
    }

}
