package nl.iobyte.themeparkconnector.api.network.drivers;

import io.socket.client.Manager;
import io.socket.client.Socket;
import nl.iobyte.themeparkconnector.ThemeParkConnector;
import nl.iobyte.themeparkconnector.api.ThemeParkConnectorAPI;
import nl.iobyte.themeparkconnector.api.client.ClientConnection;
import nl.iobyte.themeparkconnector.api.message.MessageKey;
import nl.iobyte.themeparkconnector.api.network.io.SocketConnector;
import nl.iobyte.themeparkconnector.api.network.objects.SocketDriver;
import nl.iobyte.themeparkconnector.api.state.states.ConnectedState;
import nl.iobyte.themeparkconnector.api.state.states.IdleState;
import nl.iobyte.themeparkconnector.logger.ThemeParkConnectorLogger;

import java.util.Arrays;

public class SystemDriver extends SocketDriver {

    /**
     * Register what to do whit connection
     * @param socket Socket
     * @param connector SocketConnector
     */
    public void boot(Socket socket, SocketConnector connector) {
        //What to do when connecting
        socket.on(Socket.EVENT_CONNECT, args -> {
            ThemeParkConnectorAPI api = ThemeParkConnector.getInstance().getAPI();
            api.getStateService().setState(new ConnectedState());
            api.getPlayerStateService().start();

            for(ClientConnection client : ThemeParkConnector.getInstance().getAPI().getNetworkingService().getClients()) {
                if(!client.isWaitingForToken())
                    continue;

                client.publishURL();
            }
        });

        //What to do when losing connection
        socket.on(Socket.EVENT_DISCONNECT, args -> {
            if(ThemeParkConnector.getInstance() == null || ThemeParkConnector.getInstance().isDisabling())
                return;

            cancelWaitForToken();
            ThemeParkConnectorAPI api = ThemeParkConnector.getInstance().getAPI();
            if(!(api.getStateService().getCurrentState() instanceof IdleState))
                api.getStateService().setState(new IdleState("Disconnected from socket"));

            api.getPlayerStateService().stop();
            for(ClientConnection client : ThemeParkConnector.getInstance().getAPI().getNetworkingService().getClients()) {
                if(!client.isConnected())
                    continue;

                client.onDisconnect();
            }
        });

        //What to do when can't connect
        socket.on(Socket.EVENT_CONNECT_ERROR, args -> {
            cancelWaitForToken();
            ThemeParkConnectorAPI api = ThemeParkConnector.getInstance().getAPI();
            api.getStateService().setState(new IdleState("Socket connection error"));
            api.getPlayerStateService().stop();

            Arrays.stream(args).filter(arg -> {
                if(!(arg instanceof Throwable))
                    return false;

                ((Throwable) arg).printStackTrace();
                return false;
            }).forEach(arg -> ThemeParkConnectorLogger.toConsole(ThemeParkConnector.getGson().toJson(arg)));
        });

        socket.on(Manager.EVENT_ERROR, args -> Arrays.stream(args).filter(arg -> {
            if(!(arg instanceof Throwable))
                return false;

            ((Throwable) arg).printStackTrace();
            return false;
        }).forEach(arg -> ThemeParkConnectorLogger.toConsole(ThemeParkConnector.getGson().toJson(arg))));
    }

    /**
     * Cancel tokens for all people waiting for one
     */
    private void cancelWaitForToken() {
        if(ThemeParkConnector.getInstance() == null)
            return;

        for(ClientConnection client : ThemeParkConnector.getInstance().getAPI().getNetworkingService().getClients()) {
            if(!client.isWaitingForToken())
                continue;

            client.setWaitingForToken(false);
            MessageKey.CLIENT_UNABLE_TO_CONNECT.send(client.getPlayer());
        }
    }

}
